@@ -26,7 +26,6 @@ package com.oracle.svm.core.deopt;
 
 import java.lang.ref.WeakReference;
 
-import org.graalvm.compiler.nodes.FrameState;
 import org.graalvm.nativeimage.PinnedObject;
 import org.graalvm.nativeimage.c.function.CodePointer;
 import org.graalvm.word.Pointer;
@@ -48,6 +47,7 @@ import com.oracle.svm.core.log.StringBuilderLog;
 import com.oracle.svm.core.meta.SubstrateObjectConstant;
 import com.oracle.svm.core.monitor.MonitorSupport;
 
+import jdk.graal.compiler.nodes.FrameState;
 import jdk.vm.ci.code.InstalledCode;
 import jdk.vm.ci.meta.JavaConstant;
 
@@ -410,9 +410,10 @@ public final class DeoptimizedFrame {
             return;
         }
         ReturnAddress firstAddressEntry = topFrame.returnAddress;
-        CodeInfo info = CodeInfoTable.getImageCodeInfo();
+        CodePointer ip = WordFactory.pointer(firstAddressEntry.returnAddress);
+        CodeInfo info = CodeInfoTable.getImageCodeInfo(ip);
         SimpleCodeInfoQueryResult codeInfoQueryResult = UnsafeStackValue.get(SimpleCodeInfoQueryResult.class);
-        CodeInfoAccess.lookupCodeInfo(info, CodeInfoAccess.relativeIP(info, WordFactory.pointer(firstAddressEntry.returnAddress)), codeInfoQueryResult);
+        CodeInfoAccess.lookupCodeInfo(info, ip, codeInfoQueryResult);
         long handler = codeInfoQueryResult.getExceptionOffset();
         if (handler == 0) {
             throwMissingExceptionHandler(info, firstAddressEntry);
@@ -424,7 +425,7 @@ public final class DeoptimizedFrame {
     @RestrictHeapAccess(access = RestrictHeapAccess.Access.UNRESTRICTED, reason = "Printing out error and then crashing.")
     private static void throwMissingExceptionHandler(CodeInfo info, ReturnAddress firstAddressEntry) {
         CodeInfoQueryResult detailedQueryResult = new CodeInfoQueryResult();
-        CodeInfoAccess.lookupCodeInfo(info, CodeInfoAccess.relativeIP(info, WordFactory.pointer(firstAddressEntry.returnAddress)), detailedQueryResult);
+        CodeInfoAccess.lookupCodeInfo(info, WordFactory.pointer(firstAddressEntry.returnAddress), detailedQueryResult);
         FrameInfoQueryResult frameInfo = detailedQueryResult.getFrameInfo();
         throw Deoptimizer.fatalDeoptimizationError("No exception handler registered for deopt target", frameInfo);
     }
