@@ -29,12 +29,10 @@ import static com.oracle.svm.core.jfr.Target_jdk_jfr_internal_JVM_Util.jfrNotSup
 import java.util.List;
 import java.util.function.BooleanSupplier;
 
-import org.graalvm.compiler.api.replacements.Fold;
 import org.graalvm.nativeimage.Platform;
 import org.graalvm.nativeimage.Platforms;
 import org.graalvm.nativeimage.ProcessProperties;
 
-import com.oracle.svm.core.Containers;
 import com.oracle.svm.core.Uninterruptible;
 import com.oracle.svm.core.VMInspectionOptions;
 import com.oracle.svm.core.annotate.Alias;
@@ -42,11 +40,17 @@ import com.oracle.svm.core.annotate.RecomputeFieldValue;
 import com.oracle.svm.core.annotate.Substitute;
 import com.oracle.svm.core.annotate.TargetClass;
 import com.oracle.svm.core.annotate.TargetElement;
+import com.oracle.svm.core.container.Container;
+import com.oracle.svm.core.container.OperatingSystem;
+import com.oracle.svm.core.jdk.JDKLatest;
 import com.oracle.svm.core.jfr.traceid.JfrTraceId;
+import com.oracle.svm.core.util.PlatformTimeUtils;
 import com.oracle.svm.util.ReflectionUtil;
 
+import jdk.graal.compiler.api.replacements.Fold;
 import jdk.jfr.internal.JVM;
 import jdk.jfr.internal.LogTag;
+import jdk.jfr.internal.event.EventWriter;
 
 /**
  * The substitutions below are always active, even if the JFR support is disabled. Otherwise, we
@@ -67,15 +71,18 @@ public final class Target_jdk_jfr_internal_JVM {
 
     @Alias //
     @RecomputeFieldValue(kind = RecomputeFieldValue.Kind.Reset) //
-    private volatile boolean nativeOK;
+    @TargetElement(onlyWith = JDKLatest.class) //
+    private static volatile boolean nativeOK;
 
     /** See {@link JVM#registerNatives}. */
     @Substitute
     private static void registerNatives() {
     }
 
+    /** See {@link JVM#markChunkFinal}. */
     @Substitute
-    public void markChunkFinal() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void markChunkFinal() {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -84,7 +91,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#beginRecording}. */
     @Substitute
-    public void beginRecording() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void beginRecording() {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -94,7 +102,8 @@ public final class Target_jdk_jfr_internal_JVM {
     /** See {@link JVM#isRecording}. */
     @Substitute
     @Uninterruptible(reason = "Needed for calling SubstrateJVM.isRecording().")
-    public boolean isRecording() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static boolean isRecording() {
         if (!HasJfrSupport.get()) {
             return false;
         }
@@ -103,7 +112,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#endRecording}. */
     @Substitute
-    public void endRecording() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void endRecording() {
         if (!HasJfrSupport.get()) {
             /* Nothing to do. */
             return;
@@ -119,19 +129,22 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#emitEvent}. */
     @Substitute
-    public boolean emitEvent(long eventTypeId, long timestamp, long when) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static boolean emitEvent(long eventTypeId, long timestamp, long when) {
         return false;
     }
 
     /** See {@link JVM#getAllEventClasses}. */
     @Substitute
-    public List<Class<? extends jdk.internal.event.Event>> getAllEventClasses() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static List<Class<? extends jdk.internal.event.Event>> getAllEventClasses() {
         return JfrJavaEvents.getAllEventClasses();
     }
 
     /** See {@link JVM#getUnloadedEventClassCount}. */
     @Substitute
-    public long getUnloadedEventClassCount() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static long getUnloadedEventClassCount() {
         return 0;
     }
 
@@ -152,7 +165,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#getPid}. */
     @Substitute
-    public String getPid() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static String getPid() {
         long id = ProcessProperties.getProcessID();
         return String.valueOf(id);
     }
@@ -160,7 +174,8 @@ public final class Target_jdk_jfr_internal_JVM {
     /** See {@link JVM#getStackTraceId}. */
     @Substitute
     @Uninterruptible(reason = "Needed for SubstrateJVM.getStackTraceId().")
-    public long getStackTraceId(int skipCount) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static long getStackTraceId(int skipCount, long stackFilterId) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -172,9 +187,32 @@ public final class Target_jdk_jfr_internal_JVM {
         return SubstrateJVM.get().getStackTraceId(skipCount);
     }
 
+    @Substitute
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static long registerStackFilter(String[] classes, String[] methods) {
+        throw new UnsupportedOperationException("JFR stack filters are not supported at the moment.");
+    }
+
+    @Substitute
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void unregisterStackFilter(long stackFilterId) {
+        /* Ignore the call for now (registerStackFilter() is not implemented). */
+    }
+
+    /**
+     * As of 22+27, This method is both used to set cutoff tick values for leak profiling and
+     * for @Deprecated events. Note that this method is called during JFR startup.
+     */
+    @Substitute
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setMiscellaneous(long eventTypeId, long value) {
+        /* Ignore the call and don't throw an exception (would result in an unspecific warning). */
+    }
+
     /** See {@link JVM#getThreadId}. */
     @Substitute
-    public long getThreadId(Thread t) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static long getThreadId(Thread t) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -183,10 +221,18 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#getTicksFrequency}. */
     @Substitute
-    public long getTicksFrequency() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static long getTicksFrequency() {
         return JfrTicks.getTicksFrequency();
     }
 
+    /** See {@code JVM#nanosNow}. */
+    @Substitute
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static long nanosNow() {
+        return PlatformTimeUtils.singleton().nanosNow();
+    }
+
     /** See {@link JVM#log}. */
     @Substitute
     public static void log(int tagSetId, int level, String message) {
@@ -216,13 +262,15 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#retransformClasses}. */
     @Substitute
-    public synchronized void retransformClasses(Class<?>[] classes) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static synchronized void retransformClasses(Class<?>[] classes) {
         // Not supported but this method is called during JFR startup, so we can't throw an error.
     }
 
     /** See {@link JVM#setEnabled}. */
     @Substitute
-    public void setEnabled(long eventTypeId, boolean enabled) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setEnabled(long eventTypeId, boolean enabled) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -231,7 +279,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setFileNotification}. */
     @Substitute
-    public void setFileNotification(long delta) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setFileNotification(long delta) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -240,7 +289,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setGlobalBufferCount}. */
     @Substitute
-    public void setGlobalBufferCount(long count) throws IllegalArgumentException, IllegalStateException {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setGlobalBufferCount(long count) throws IllegalArgumentException, IllegalStateException {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -249,7 +299,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setGlobalBufferSize}. */
     @Substitute
-    public void setGlobalBufferSize(long size) throws IllegalArgumentException {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setGlobalBufferSize(long size) throws IllegalArgumentException {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -258,7 +309,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setMemorySize}. */
     @Substitute
-    public void setMemorySize(long size) throws IllegalArgumentException {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setMemorySize(long size) throws IllegalArgumentException {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -267,7 +319,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@code JVM#setMethodSamplingPeriod}. */
     @Substitute
-    public void setMethodSamplingPeriod(long type, long intervalMillis) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setMethodSamplingPeriod(long type, long intervalMillis) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -276,7 +329,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setOutput}. */
     @Substitute
-    public void setOutput(String file) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setOutput(String file) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -285,12 +339,14 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setForceInstrumentation}. */
     @Substitute
-    public void setForceInstrumentation(boolean force) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setForceInstrumentation(boolean force) {
     }
 
     /** See {@link JVM#setCompressedIntegers}. */
     @Substitute
-    public void setCompressedIntegers(boolean compressed) throws IllegalStateException {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setCompressedIntegers(boolean compressed) throws IllegalStateException {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -299,7 +355,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setStackDepth}. */
     @Substitute
-    public void setStackDepth(int depth) throws IllegalArgumentException, IllegalStateException {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setStackDepth(int depth) throws IllegalArgumentException, IllegalStateException {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -308,7 +365,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setStackTraceEnabled}. */
     @Substitute
-    public void setStackTraceEnabled(long eventTypeId, boolean enabled) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setStackTraceEnabled(long eventTypeId, boolean enabled) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -317,7 +375,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setThreadBufferSize}. */
     @Substitute
-    public void setThreadBufferSize(long size) throws IllegalArgumentException, IllegalStateException {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setThreadBufferSize(long size) throws IllegalArgumentException, IllegalStateException {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -326,7 +385,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setThreshold}. */
     @Substitute
-    public boolean setThreshold(long eventTypeId, long ticks) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static boolean setThreshold(long eventTypeId, long ticks) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -335,7 +395,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#storeMetadataDescriptor}. */
     @Substitute
-    public void storeMetadataDescriptor(byte[] bytes) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void storeMetadataDescriptor(byte[] bytes) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -344,13 +405,15 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#getAllowedToDoEventRetransforms}. */
     @Substitute
-    public boolean getAllowedToDoEventRetransforms() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static boolean getAllowedToDoEventRetransforms() {
         return false;
     }
 
     /** See {@link JVM#createJFR}. */
     @Substitute
-    private boolean createJFR(boolean simulateFailure) throws IllegalStateException {
+    @TargetElement(onlyWith = JDKLatest.class)
+    private static boolean createJFR(boolean simulateFailure) throws IllegalStateException {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -359,7 +422,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#destroyJFR}. */
     @Substitute
-    private boolean destroyJFR() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    private static boolean destroyJFR() {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -368,19 +432,22 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#isAvailable}. */
     @Substitute
-    public boolean isAvailable() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static boolean isAvailable() {
         return HasJfrSupport.get();
     }
 
     /** See {@link JVM#getTimeConversionFactor}. */
     @Substitute
-    public double getTimeConversionFactor() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static double getTimeConversionFactor() {
         return 1;
     }
 
     /** See {@link JVM#getTypeId(Class)}. */
     @Substitute
-    public long getTypeId(Class<?> clazz) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static long getTypeId(Class<?> clazz) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -389,7 +456,7 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#getEventWriter}. */
     @Substitute
-    public static Object getEventWriter() {
+    public static Target_jdk_jfr_internal_event_EventWriter getEventWriter() {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -405,7 +472,7 @@ public final class Target_jdk_jfr_internal_JVM {
         return SubstrateJVM.get().newEventWriter();
     }
 
-    /** See {@link JVM#flush}. */
+    /** See {@link JVM#flush(EventWriter, int, int)}. */
     @Substitute
     public static void flush(Target_jdk_jfr_internal_event_EventWriter writer, int uncommittedSize, int requestedSize) {
         if (!HasJfrSupport.get()) {
@@ -414,14 +481,17 @@ public final class Target_jdk_jfr_internal_JVM {
         SubstrateJVM.get().flush(writer, uncommittedSize, requestedSize);
     }
 
+    /** See {@link JVM#flush()}. */
     @Substitute
-    public void flush() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void flush() {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
         SubstrateJVM.get().flush();
     }
 
+    /** See {@link JVM#commit}. */
     @Substitute
     public static long commit(long nextPosition) {
         if (!HasJfrSupport.get()) {
@@ -432,7 +502,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setRepositoryLocation}. */
     @Substitute
-    public void setRepositoryLocation(String dirText) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setRepositoryLocation(String dirText) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -441,7 +512,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@code JVM#setDumpPath(String)}. */
     @Substitute
-    public void setDumpPath(String dumpPathText) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void setDumpPath(String dumpPathText) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -450,7 +522,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@code JVM#getDumpPath()}. */
     @Substitute
-    public String getDumpPath() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static String getDumpPath() {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -459,7 +532,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#abort}. */
     @Substitute
-    public void abort(String errorMsg) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void abort(String errorMsg) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -474,7 +548,8 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#uncaughtException}. */
     @Substitute
-    public void uncaughtException(Thread thread, Throwable t) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void uncaughtException(Thread thread, Throwable t) {
         /*
          * Would be used to determine the emergency dump filename if an exception happens during
          * shutdown.
@@ -483,28 +558,38 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#setCutoff}. */
     @Substitute
-    public boolean setCutoff(long eventTypeId, long cutoffTicks) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static boolean setCutoff(long eventTypeId, long cutoffTicks) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
         return SubstrateJVM.get().setCutoff(eventTypeId, cutoffTicks);
     }
 
+    /** See {@link JVM#setThrottle}. */
     @Substitute
-    public boolean setThrottle(long eventTypeId, long eventSampleSize, long periodMs) {
-        // Not supported but this method is called during JFR startup, so we can't throw an error.
-        return true;
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static boolean setThrottle(long eventTypeId, long eventSampleSize, long periodMs) {
+        if (!HasJfrSupport.get()) {
+            throw jfrNotSupportedException();
+        }
+        return SubstrateJVM.get().setThrottle(eventTypeId, eventSampleSize, periodMs);
     }
 
     /** See {@link JVM#emitOldObjectSamples}. */
     @Substitute
-    public void emitOldObjectSamples(long cutoff, boolean emitAll, boolean skipBFS) {
-        // Not supported but this method is called during JFR shutdown, so we can't throw an error.
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void emitOldObjectSamples(long cutoff, boolean emitAll, boolean skipBFS) {
+        if (!HasJfrSupport.get()) {
+            throw jfrNotSupportedException();
+        }
+        SubstrateJVM.get().emitOldObjectSamples(cutoff, emitAll, skipBFS);
     }
 
     /** See {@link JVM#shouldRotateDisk}. */
     @Substitute
-    public boolean shouldRotateDisk() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static boolean shouldRotateDisk() {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -513,37 +598,46 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#include}. */
     @Substitute
-    public void include(Thread thread) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void include(Thread thread) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
         JfrThreadLocal.setExcluded(thread, false);
     }
 
+    /** See {@link JVM#exclude}. */
     @Substitute
-    public void exclude(Thread thread) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static void exclude(Thread thread) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
         JfrThreadLocal.setExcluded(thread, true);
     }
 
+    /** See {@link JVM#isExcluded(Thread)}. */
     @Substitute
-    public boolean isExcluded(Thread thread) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static boolean isExcluded(Thread thread) {
         if (!HasJfrSupport.get()) {
             return true;
         }
         return JfrThreadLocal.isThreadExcluded(thread);
     }
 
+    /** See {@link JVM#isExcluded(Class)}. */
     @Substitute
-    public boolean isExcluded(Class<? extends jdk.internal.event.Event> eventClass) {
+    @TargetElement(onlyWith = JDKLatest.class) //
+    public static boolean isExcluded(Class<? extends jdk.internal.event.Event> eventClass) {
         /* For now, assume that event classes are only excluded if JFR support is disabled. */
         return !HasJfrSupport.get();
     }
 
+    /** See {@link JVM#isInstrumented}. */
     @Substitute
-    public boolean isInstrumented(Class<? extends jdk.internal.event.Event> eventClass) {
+    @TargetElement(onlyWith = JDKLatest.class) //
+    public static boolean isInstrumented(Class<? extends jdk.internal.event.Event> eventClass) {
         /*
          * Assume that event classes are instrumented if JFR support is present. This method should
          * ideally check for blessed commit methods in the event class, see GR-41200.
@@ -551,25 +645,30 @@ public final class Target_jdk_jfr_internal_JVM {
         return HasJfrSupport.get();
     }
 
-    /** See {@link SubstrateJVM#getChunkStartNanos}. */
+    /** See {@link JVM#getChunkStartNanos}. */
     @Substitute
-    public long getChunkStartNanos() {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static long getChunkStartNanos() {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
         return SubstrateJVM.get().getChunkStartNanos();
     }
 
+    /** See {@link JVM#setConfiguration}. */
     @Substitute
-    public boolean setConfiguration(Class<? extends jdk.internal.event.Event> eventClass, Target_jdk_jfr_internal_event_EventConfiguration configuration) {
+    @TargetElement(onlyWith = JDKLatest.class) //
+    public static boolean setConfiguration(Class<? extends jdk.internal.event.Event> eventClass, Target_jdk_jfr_internal_event_EventConfiguration configuration) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
         return SubstrateJVM.get().setConfiguration(eventClass, configuration);
     }
 
+    /** See {@link JVM#getConfiguration}. */
     @Substitute
-    public Object getConfiguration(Class<? extends jdk.internal.event.Event> eventClass) {
+    @TargetElement(onlyWith = JDKLatest.class) //
+    public static Object getConfiguration(Class<? extends jdk.internal.event.Event> eventClass) {
         if (!HasJfrSupport.get()) {
             throw jfrNotSupportedException();
         }
@@ -578,18 +677,34 @@ public final class Target_jdk_jfr_internal_JVM {
 
     /** See {@link JVM#getTypeId(String)}. */
     @Substitute
-    public long getTypeId(String name) {
+    @TargetElement(onlyWith = JDKLatest.class)
+    public static long getTypeId(String name) {
         /* Not implemented at the moment. */
         return -1;
     }
 
+    /** See {@link JVM#isContainerized}. */
     @Substitute
-    public boolean isContainerized() {
-        return Containers.isContainerized();
+    @TargetElement(onlyWith = JDKLatest.class) //
+    public static boolean isContainerized() {
+        return Container.singleton().isContainerized();
+    }
+
+    /**
+     * See {@link JVM#hostTotalMemory()}.
+     * <p>
+     * This calls {@link OperatingSystem#getPhysicalMemorySize} since we are interested in the host
+     * values (and not the containerized values).
+     */
+    @Substitute
+    @TargetElement(onlyWith = JDKLatest.class) //
+    public static long hostTotalMemory() {
+        return OperatingSystem.singleton().getPhysicalMemorySize().rawValue();
     }
 
     @Substitute
-    public long hostTotalMemory() {
+    @TargetElement(onlyWith = JDKLatest.class) //
+    public static long hostTotalSwapMemory() {
         /* Not implemented at the moment. */
         return -1;
     }
